﻿//
//  ICFrame.as
//  InfiniteCanvas (Flash Viewer)
//
//  Created by Markus Mueller <markus@mindnode.com>.
//  http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

class ICFrame {
	// Represents a Navigation Frame or Active Component.
	
	private var _x:Number = 0; 
	private var _y:Number = 0;
	private var _width:Number = 0;
	private var _height:Number = 0;
	private var _rotation:Number = 0;
	private var _id: Number = 0;
	
	private var _hotSpots:Array = null;
	private var _navigationTrails:Array = null;
	
	private var _link:String = null;
	private var _linkTarget:String = null;
	private var _delay:Number = -1;

	// constructor
	function ICFrame(id:Number, x:Number, y:Number, w:Number, h:Number, r:Number) {
		_navigationTrails = new Array();
		_hotSpots = new Array();

		_id = id;
		_x = x;
		_y = y;
		_width = w;
		_height = h;
		_rotation = r;
	
		_link = "";
		_linkTarget = "_self";
	}
	
	public function addHotSpot(x:Number, y:Number, w:Number, h:Number):ICFrame {
		var newFrame:ICFrame = new ICFrame(-1, x, y, w, h);
		_hotSpots.push(newFrame);
		
		return newFrame;
	}
	
	public function addNavigationTrail(trail:ICNavigationTrail) {
		_navigationTrails.push(trail);
	}
	
	public function getX():Number {
		return _x;
	}
	
	public function getY():Number {
		return _y;
	}
		
	public function getWidth():Number {
		return _width;
	}
	
	public function getHeight():Number {
		return _height;
	}
	
	public function getRotation():Number {
		return _rotation;
	}
	
	public function getID():Number {
		return _id;
	}
	
	public function getDelay():Number {
		return _delay;
	}
	
	public function setDelay(delay:Number) {
		_delay = delay;
	}
	
	/*public function checkActiveComponentHit(x:Number, y:Number) : ICFrame {		
		for (var i:Number = 0; i < _activeComponents.length; i++) {
			if (_activeComponents[i].hitTest(x, y)) {
				return _activeComponents[i];
			}
		}
		return null;
	}*/
	
	public function getHotSpots() {
		return _hotSpots;
	}
	
	public function getNavigationTrails() {
		return _navigationTrails;
	}
	
	public function getTrail():ICNavigationTrail {
		// return a trail. one will be picked my random if there are more then one trails.
		if (_navigationTrails.length == 0) {
			return null;
		} else if (_navigationTrails.length == 1) {
			return _navigationTrails[0];
		} else {
			var randomNumber:Number = Math.round(Math.random()*_navigationTrails.length); 
			return _navigationTrails[randomNumber];
		}
	}
	
	public function getTrailCount() : Number {
		return _navigationTrails.length;
	}
	
	function hitTest(x:Number, y:Number) : Boolean {
		if ((_y + _height) < y) return false;
		
		if (_y > y) return false;
		
		if ((_x + _width) < x) return false;
		
		if (_x > x) return false;
		
		return true;
	}

	public function openURL() : Boolean {
		if ((_link != null) && (_link.length > 0)) {
			getURL(_link, _linkTarget);
			return true;
		} else {
			return false;
		}
	}
	
	public function setURL(newLink:String, newTarget:String) {
		_link = newLink;
		_linkTarget = newTarget
	}
}