﻿//
//  ICParser.as
//  InfiniteCanvas (Flash Viewer)
//
//  Created by Markus Mueller <markus@mindnode.com>.
//  http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

import de.alex_uhlmann.animationpackage.*;
import de.alex_uhlmann.animationpackage.animation.*;
import de.alex_uhlmann.animationpackage.drawing.*;
import de.alex_uhlmann.animationpackage.utility.*;
import com.robertpenner.easing.*;

class ICParser {
	// This class parses the XML document
	
	private var _xmlDocument:XML = null;
	private var _document:ICDocument = null;
	private var _path:String = null;
	private var _controller:ICController = null;
	
	// Constructor
	function ICParser(filePath:String, controller:ICController) {
		_controller = controller;
		// init animation package
		APCore.initialize();

		// set path
		_path = filePath;
		
		// create new XML object
		_xmlDocument = new XML();
		_xmlDocument.ignoreWhite = true;
		
		var owner:ICParser = this;

		_xmlDocument.onLoad = function(sucess:Boolean) {
        	if (sucess) {
                owner.parseDocument(this);
				owner._controller.checkIfReady();
       	 	} else {
				_controller.printLoadingErrorMessage("Failed to load XML file!");
			}
		};
		
		var tmpPath:String;
		if (_path.length > 0) {
			tmpPath = _path.charAt(_path.length - 1);
			if (tmpPath != "/") {
				_path = _path + "/";
			}
		}
		_xmlDocument.load(_path + "comic.xml");
	}

	// creates a new ICDocument
	function parseDocument(xmlTree:XML) : Void {
		// first child = ic
		if (xmlTree.firstChild.attributes.version == "1.3") {
			_document = new ICDocument();
			if (!this.initDocumentFromXML(xmlTree.firstChild)) {
				_document = null;
				_controller.printLoadingErrorMessage("Failed to parse XML file!");
			} else {
				//_controller.printLoadingErrorMessage("Comic sucessfully parsed!");
			}
		} else {
			_controller.printLoadingErrorMessage("Can't parse this Infinite Canvas Document!");
		}
		
		// free some memory
		_xmlDocument.removeNode();
		_xmlDocument = null;
	}
	
	
	public function initDocumentFromXML(icNode:XMLNode):Boolean {
		for (var i = 0; i < icNode.childNodes.length; i++) {
			if (icNode.childNodes[i].nodeName == "canvas") {
				_controller._loadCount = icNode.childNodes[i].childNodes.length+1;
				// +1 because xml/trails are counted as 1 MC

				// set the canvas color		
				_controller._canvasBG.moveTo(0, 0);
				_controller._canvasBG.beginFill(icNode.childNodes[i].attributes.color)
				_controller._canvasBG.lineTo(_controller._canvasWidth, 0);
				_controller._canvasBG.lineTo(_controller._canvasWidth, _controller._canvasHeight);
				_controller._canvasBG.lineTo(0, _controller._canvasHeight);
				_controller._canvasBG.endFill();				
				
				// extract the canvas objects
				for (var j = 0; j < icNode.childNodes[i].childNodes.length; j++) {
					if (icNode.childNodes[i].childNodes[j].nodeName == "panel") {
						var tmpClip:ICMovieClip = this.initPanelMovieClipFromXML(icNode.childNodes[i].childNodes[j]);
						if (tmpClip != null) {
							// add to document
							_document.addCanvasObject(tmpClip);
						} else {
							_controller.printLoadingErrorMessage("Couldn't load panels!");
						}
					} else if (icNode.childNodes[i].childNodes[j].nodeName == "trail") {
						var tmpClip:ICMovieClip = this.initTrailMovieClipFromXML(icNode.childNodes[i].childNodes[j]);
						if (tmpClip != null) {
							// don't count trails as movies
							_controller._loadCount--; 
							tmpClip.initTrail();
							_document.addCanvasObject(tmpClip);
						} else {
							_controller.printLoadingErrorMessage("Couldn't create trail!");
						}
					}
				}
			} else if (icNode.childNodes[i].nodeName == "navigation") {
				// extract attributes
				_document.setFirstFrame(Number(icNode.childNodes[i].attributes.firstframe));
				_document.setNavigationMode(Number(icNode.childNodes[i].attributes.mode));
				_document.setHotSpotHint((icNode.childNodes[i].attributes.hotspothint == "true") ? true : false);
				_document.setScaleCanvas((icNode.childNodes[i].attributes.scale == "true") ? true : false);

				// extract navigation object
				for (var j = 0; j < icNode.childNodes[i].childNodes.length; j++) {
					if (icNode.childNodes[i].childNodes[j].nodeName == "frame") {
						var tmpFrame:ICFrame = initFrameFromXML(icNode.childNodes[i].childNodes[j]);
						if (tmpFrame != null) {
							// add to navigation object
							_document.addNavigationObject(tmpFrame); 
						}
					} 
				}
			}
		}
		_controller._loadStatus++;
		_controller.updateProgressBar();
		return true;
	}
	
	
	private function initPanelMovieClipFromXML(objectNode:XMLNode):ICMovieClip {
		// load swf
		var depth = _controller._canvas.getNextHighestDepth();
		var objectClip = _controller._canvas.attachMovie("ICMovieClip", depth, depth);
		
		// create container mc
		objectClip.createEmptyMovieClip("c",1);
		objectClip.setContainer(objectClip.c);


		if (_controller._loader.loadClip(_path + objectNode.attributes.file, objectClip.c)) {
			objectClip.setCanvasX(Number(objectNode.attributes.x));
			objectClip.setCanvasY(Number(objectNode.attributes.y));
			objectClip.setCanvasWidth(Number(objectNode.attributes.w));
			objectClip.setCanvasHeight(Number(objectNode.attributes.h));
			objectClip.setCanvasRotation(Number(objectNode.attributes.r));
			objectClip.setCanvasTransparency(Number(objectNode.attributes.t));
			return objectClip;
		} else {
			trace("jjj");
			delete objectClip;
			return null;
		}
	}
	
	private function initTrailMovieClipFromXML(objectNode:XMLNode):ICMovieClip {
		var depth = _controller._canvas.getNextHighestDepth();
		var objectClip = _controller._canvas.attachMovie("ICMovieClip", depth, depth);
		
		// container clip
		//objectClip.createEmptyMovieClip("c",1);
		//objectClip.setContainer(objectClip.c);

		// create trail mc
		var trail:MovieClip = objectClip.createEmptyMovieClip("t",1);

		var x:Number = Number(objectNode.attributes.x);
		var y:Number = Number(objectNode.attributes.y);
		var penX:Number = 0;
		var penY:Number = 0;
		objectClip.setCanvasX(x);
		objectClip.setCanvasY(y);
		var thickness:Number = Number(objectNode.attributes.thickness)
		var color:Number = objectNode.attributes.color;
		trail.lineStyle(thickness, color, 100);
		trail.moveTo(penX,penY);

		// create trail
		for (var k = 0; k < objectNode.childNodes.length; k++) {
			if (objectNode.childNodes[k].nodeName == "line") {
				penX = Number(objectNode.childNodes[k].attributes.x) - x;
				penY = Number(objectNode.childNodes[k].attributes.y) - y;
				trail.lineTo(penX, penY);
			} else if (objectNode.childNodes[k].nodeName == "curve") {
				var trailDepth = objectClip.getNextHighestDepth();
				var curve:MovieClip = objectClip.createEmptyMovieClip(trailDepth, trailDepth);	

				var oldPenX:Number = penX;
				var oldPenY:Number = penY;
				penX = Number(objectNode.childNodes[k].attributes.x) - x;
				penY = Number(objectNode.childNodes[k].attributes.y) - y;
				
				var c2X:Number = Number(objectNode.childNodes[k].attributes.c2X) - x;
				var c2Y:Number = Number(objectNode.childNodes[k].attributes.c2Y) - y;

				if ((c2X != oldPenX) && (c2Y != oldPenY)) {
					var newCurve:CubicCurve = new CubicCurve(curve, oldPenX, oldPenY, 
												Number(objectNode.childNodes[k].attributes.c1X) - x, 
												Number(objectNode.childNodes[k].attributes.c1Y) - y,
												c2X, c2Y, penX, penY, true);
					newCurve.lineStyle(thickness, color, 100);
					newCurve.draw();
				} else {
					var newCurve:QuadCurve = new QuadCurve(curve, oldPenX, oldPenY, 
												Number(objectNode.childNodes[k].attributes.c1X) - x, 
												Number(objectNode.childNodes[k].attributes.c1Y) - y,
												penX, penY, true);
					newCurve.lineStyle(thickness, color, 100);
					newCurve.draw();
				}
				
				
				// make sure that pen is at the right x/y if a line sequment follows
				trail.moveTo(penX,penY); 
			}
		}
		return objectClip;
	}
	
	private function initFrameFromXML(objectNode:XMLNode):ICFrame {
		var newFrame:ICFrame = new ICFrame(Number(objectNode.attributes.id), 
										   Number(objectNode.attributes.x), 
										   Number(objectNode.attributes.y), 
										   Number(objectNode.attributes.w), 
										   Number(objectNode.attributes.h),
										   Number(objectNode.attributes.r));
		
		
		for (var j = 0; j < objectNode.childNodes.length; j++) {
			if (objectNode.childNodes[j].nodeName == "hotspot") {
				var newHotSpot:ICFrame = newFrame.addHotSpot(Number(objectNode.childNodes[j].attributes.x),
													   Number(objectNode.childNodes[j].attributes.y),
													   Number(objectNode.childNodes[j].attributes.w),
													   Number(objectNode.childNodes[j].attributes.h));	
				for (var k = 0; k < objectNode.childNodes[j].childNodes.length; k++) {
					if (objectNode.childNodes[j].childNodes[k].nodeName == "trail") {
						newHotSpot.addNavigationTrail(initNavigationTrailFromXML(objectNode.childNodes[j].childNodes[k]));
					}
				}
			} else if (objectNode.childNodes[j].nodeName == "trail") {
				newFrame.addNavigationTrail(initNavigationTrailFromXML(objectNode.childNodes[j]));
			} else if (objectNode.childNodes[j].nodeName == "url") {
				newFrame.setURL(objectNode.childNodes[j].attributes.link, objectNode.childNodes[j].attributes.target);				
			} else if (objectNode.childNodes[j].nodeName == "transition") {
				newFrame.setDelay(objectNode.childNodes[j].attributes.delay);
			}			
		}
		return newFrame;
	}
	
	private function initNavigationTrailFromXML(objectNode:XMLNode):ICNavigationTrail {
		var newTrail:ICNavigationTrail = new ICNavigationTrail(objectNode.attributes.type,
															   Number(objectNode.attributes.target),
															   Number(objectNode.attributes.speed));
		
			// fade transitions require a color
			if (objectNode.attributes.type == "fade") {
				newTrail.setColor(objectNode.attributes.color);
			}
			
			newTrail.setEasing(objectNode.attributes.easing);
		
			for (var l = 0; l < objectNode.childNodes.length; l++) {
				if (objectNode.childNodes[l].nodeName == "line") {
					newTrail.addLine(Number(objectNode.childNodes[l].attributes.x),
									Number(objectNode.childNodes[l].attributes.y));
				} else 	if (objectNode.childNodes[l].nodeName == "curve") {
					newTrail.addCurve(Number(objectNode.childNodes[l].attributes.c1X),
										Number(objectNode.childNodes[l].attributes.c1Y),
										Number(objectNode.childNodes[l].attributes.c2X),
										Number(objectNode.childNodes[l].attributes.c2Y),
										Number(objectNode.childNodes[l].attributes.x),
										Number(objectNode.childNodes[l].attributes.y));
				}
			}

		return newTrail;
	}
	
	function getDocument():ICDocument {
		return _document;
	}
}